local f = CreateFrame("Frame");
local raid = {};
local debuffed = {};
local hunted = {};
local leader = "";
local inEncounter = false;
local healerAmount = 0;
local playerName = IRT_GetName("player");
local ticker = nil;

local groupIcons = {
	["1"] = "STAR",
	["2"] = "CIRCLE",
	["3"] = "DIAMOND",
	["4"] = "TRIANGLE",
};

f:RegisterEvent("PLAYER_LOGIN");
f:RegisterEvent("ENCOUNTER_START");
f:RegisterEvent("ENCOUNTER_END");
f:RegisterEvent("UNIT_AURA");

C_ChatInfo.RegisterAddonMessagePrefix("IRT_PP");

function IRT_PP_Manual()
	inEncounter = true;
	initRaid();
	debuffed = {};
	hunted = {};
	leader = IRT_GetRaidLeader();
end

local function initRaid()
	raid = {
		[1] = {},
		[2] = {},
		[3] = {},
		[4] = {},
	};
	for i = 1, 40 do
		local name, rank, group, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i);
		if (name and UnitIsConnected(name) and UnitIsVisible(name) and group < 5) then
			local role = UnitGroupRolesAssigned(name);
			if (role == "HEALER") then
				healerAmount = healerAmount + 1;
			end
			if (raid[group]) then
				raid[group][name] = {["mark"] = "", ["debuffed"] = false, ["hunted"] = false};
			end
		end
	end
	resetAssignments();
end

local function getGroup(player)
	for i = 1, 4 do
		local playerIndex = IRT_Contains(raid[i], player);
		if (playerIndex) then 
			return i;
		end
	end
end

local function assignments()
	local soakers = {
		[1] = 0,
		[2] = 0,
		[3] = 0,
		[4] = 0,
	};
	local players = {
		[1] = 0,
		[2] = 0,
		[3] = 0,
		[4] = 0,
	};
	local healers = {
		[1] = 0,
		[2] = 0,
		[3] = 0,
		[4] = 0,
	};
	local groupAssignment = {[1] = false, [2] = false, [3] = false, [4] = false};
	--Assign tanks to their group
	for group = 1, 4 do
		for player, plData in pairs(raid[group]) do
			local role = UnitGroupRolesAssigned(player);
			if (role == "TANK") then
				local mark = raid[group][player].mark;
				local isHunted = raid[group][player].hunted;
				local isDebuffed = raid[group][player].debuffed;
				if (mark == "") then 
					raid[group][player].mark = group;
					players[group] = players[group] + 1;
					if (not isDebuffed) then
						soakers[group] = soakers[group] + 1;
						IRT_SendAddonCrossRealmMessage("IRT_PP", player, "soak " .. raid[group][player].mark);
					else
						IRT_SendAddonCrossRealmMessage("IRT_PP", player, "go " .. raid[group][player].mark);
					end
				end
			else
				break;
			end
		end
	end
	--Assign player to the mark their already assigned to
	for index, player in pairs (hunted) do
		local group = getGroup(player);
		local role = UnitGroupRolesAssigned(player);
		if (groupAssignment[group] == false) then
			groupAssignment[group] = true;
			raid[group][player].hunted = true;
			raid[group][player].mark = group;
			SetRaidTarget(player, group);
			players[group] = players[group] + 1;
			if (role == "HEALER") then
				healers[group] = healers[group] + 1;
			end
			IRT_SendAddonCrossRealmMessage("IRT_PP", player, "debuff " .. raid[group][player].mark);
		end
	end
	--Assign everyone that did not get assigned already
	for index, player in pairs (hunted) do
		local group = getGroup(player);
		if (raid[group][player] and not raid[group][player].hunted) then
			for i = 1, 4 do
				local counter = group + i;
				if (counter > 4) then 
					counter = 4-group+i;
				end
				if (not groupAssignment[counter]) then 
					local role = UnitGroupRolesAssigned(player);
					if (role == "HEALER") then
						healers[group] = healers[group] + 1;
					end
					groupAssignment[counter] = true;
					raid[group][player].hunted = true;
					raid[group][player].mark = counter;
					players[counter] = players[counter] + 1;
					SetRaidTarget(player, counter);
					IRT_SendAddonCrossRealmMessage("IRT_PP", player, "debuff " .. raid[group][player].mark);
					break;
				end
			end
		end
	end
	--Assign healers to keep their group
	for group = 1, 4 do
		for player, plData in pairs(raid[group]) do
			local role = UnitGroupRolesAssigned(player);
			if (role == "HEALERS") then
				local mark = raid[group][player].mark;
				local isHunted = raid[group][player].hunted;
				local isDebuffed = raid[group][player].debuffed;
				if (mark == "" and healers[group] == 0) then 
					raid[group][player].mark = group;
					healers[group] = healers[group] + 1;
					players[group] = players[group] + 1;
					if (not isDebuffed and soakers[group] < 2) then
						soakers[group] = soakers[group] + 1;
						IRT_SendAddonCrossRealmMessage("IRT_PP", player, "soak " .. raid[group][player].mark);
					else
						IRT_SendAddonCrossRealmMessage("IRT_PP", player, "go " .. raid[group][player].mark);
					end
				end
			else
				break;
			end
		end
	end
	--Assign rest of healers
	for i = 1, 4 do
		for group = 1, 4 do
			for player, plData in pairs(raid[group]) do
				local role = UnitGroupRolesAssigned(player);
				if (role == "HEALER") then
					local mark = raid[group][player].mark;
					local isHunted = raid[group][player].hunted;
					local isDebuffed = raid[group][player].debuffed;
					if (mark == "" and healers[i] == 0) then 
						raid[group][player].mark = i;
						healers[i] = healers[i] + 1;
						players[i] = players[i] + 1;
						if (not isDebuffed and soakers[i] < 2) then
							soakers[i] = soakers[i] + 1;
							IRT_SendAddonCrossRealmMessage("IRT_PP", player, "soak " .. raid[group][player].mark);
						else
							IRT_SendAddonCrossRealmMessage("IRT_PP", player, "go " .. raid[group][player].mark);
						end
					end
				else
					break;
				end
			end
		end
	end
	--if more than 4 healers
	if (healerAmount > 4) then
		for i = healerAmount-4, 1, -1 do
			for group = 1, 4 do
				for player, plData in pairs(raid[group]) do
					local role = UnitGroupRolesAssigned(player);
					if (role == "HEALERS") then
						local mark = raid[group][player].mark;
						local isHunted = raid[group][player].hunted;
						local isDebuffed = raid[group][player].debuffed;
						if (mark == "" and healers[i] == 1) then 
							raid[group][player].mark = i;
							healers[i] = healers[i] + 1;
							players[i] = players[i] + 1;
							if (not isDebuffed and soakers[i] < 2) then
								soakers[i] = soakers[i] + 1;
								IRT_SendAddonCrossRealmMessage("IRT_PP", player, "soak " .. raid[group][player].mark);
							else
								IRT_SendAddonCrossRealmMessage("IRT_PP", player, "go " .. raid[group][player].mark);
							end
						end
					else
						break;
					end
				end
			end
		end
	end
	--Find 2 players eligible to soak per group
	for group = 1, 4 do
		for player, plData in pairs(raid[group]) do
			if (soakers[group] < 2) then
				local mark = raid[group][player].mark;
				local isHunted = raid[group][player].hunted;
				local isDebuffed = raid[group][player].debuffed;
				if (not debuffed and not isHunted and mark == "") then 
					raid[group][player].mark = group;
					soakers[group] = soakers[group] + 1;
					players[group] = players[group] + 1;
					IRT_SendAddonCrossRealmMessage("IRT_PP", player, "soak " .. raid[group][player].mark);
				end
			else
				break;
			end
		end
	end
	--Did not find 2 eligible players, find any players
	for i = 1, 4 do
		if (soakers[i] < 2) then 
			for group = 1, 4 do
				for player, plData in pairs(raid[group]) do
					if (soakers[i] < 2) then
						local mark = raid[group][player].mark;
						local isHunted = raid[group][player].hunted;
						local isDebuffed = raid[group][player].debuffed;
						if (not debuffed and not isHunted and mark == "") then 
							raid[group][player].mark = i;
							soakers[i] = soakers[i] + 1;
							players[i] = players[i] + 1;
							IRT_SendAddonCrossRealmMessage("IRT_PP", player, "soak " .. raid[group][player].mark);
						end
					else
						break;
					end
				end
			end
		end
	end
	--Fill left over players in their group their in
	for group = 1, 4 do
		for player, plData in pairs(raid[group]) do
			if (players[group] < 5) then
				local mark = raid[group][player].mark;
				local isHunted = raid[group][player].hunted;
				local isDebuffed = raid[group][player].debuffed;
				if (mark == "") then 
					raid[group][player].mark = group;
					players[group] = players[group] + 1;
					IRT_SendAddonCrossRealmMessage("IRT_PP", player, "go " .. raid[group][player].mark);
				end
			else
				break;
			end
		end
	end
	--Fill left over players in any group that can take them
	for i = 1, 4 do
		if (players[i] < 5) then 
			for group = 1, 4 do
				for player, plData in pairs(raid[group]) do
					if (players[i] < 5) then
						local mark = raid[group][player].mark;
						local isHunted = raid[group][player].hunted;
						local isDebuffed = raid[group][player].debuffed;
						if (not debuffed and not isHunted and mark == "") then 
							raid[group][player].mark = i;
							players[i] = players[i] + 1;
							IRT_SendAddonCrossRealmMessage("IRT_PP", player, "go " .. raid[group][player].mark);
						end
					else
						break;
					end
				end
			end
		end
	end
end

f:SetScript("OnEvent", function(self, event, ...)
	if (event == "PLAYER_LOGIN") then
		if (IRT_PrototypePantheonEnabled == nil) then IRT_PrototypePantheonEnabled = true; end
	elseif (event == "CHAT_MSG_ADDON" and inEncounter and IRT_PrototypePantheonEnabled) then
		local prefix, msg, channel, sender = ...;
		if (prefix == "IRT_PP") then
			local text, target = IRT_DecodeCrossRealmAddonMessage(sender, msg);
			local action, mark = strsplit(" ", text);
			sender = Ambiguate(sender, "none");
			if (target == nil or UnitIsUnit(target, playerName)) then
				if (action == "soak") then
					IRT_PopupShow("\124TInterface\\TargetingFrame\\UI-RaidTargetingIcon_"..mark..":30\124t".." SOAK " .. groupIcons[mark] .. "\124TInterface\\TargetingFrame\\UI-RaidTargetingIcon_"..mark..":30\124t", 9, L.BOSS_FILE);
					PlaySoundFile("Interface\\AddOns\\InfiniteRaidTools\\Sound\\"..groupIcons[mark]..".ogg", "Master");
					ticker = IRT_NotifyPlayer(ticker, "{"..groupIcons[mark].."} SOAK " .. "{" .. groupIcons[mark] .. "}", "SAY", 9);
				elseif (action == "go") then
					IRT_PopupShow("\124TInterface\\TargetingFrame\\UI-RaidTargetingIcon_"..mark..":30\124t".." GO TO " .. groupIcons[mark] .. ", DONT SOAK " .. "\124TInterface\\TargetingFrame\\UI-RaidTargetingIcon_"..mark..":30\124t", 9, L.BOSS_FILE);
					PlaySoundFile("Interface\\AddOns\\InfiniteRaidTools\\Sound\\"..groupIcons[mark]..".ogg", "Master");
				elseif (action == "debuff") then
					IRT_PopupShow("\124TInterface\\TargetingFrame\\UI-RaidTargetingIcon_"..mark..":30\124t".." GO TO " .. groupIcons[mark] .. "\124TInterface\\TargetingFrame\\UI-RaidTargetingIcon_"..mark..":30\124t", 9, L.BOSS_FILE);
					PlaySoundFile("Interface\\AddOns\\InfiniteRaidTools\\Sound\\"..groupIcons[mark]..".ogg", "Master");
					ticker = IRT_NotifyPlayer(ticker, "{"..groupIcons[mark].."}", "SAY", 9);
				end
			end
		end
	elseif (event == "UNIT_AURA" and IRT_PrototypePantheonEnabled and inEncounter) then
		local unit = ...;
		local unitName = IRT_GetName(unit, "true");
		local plIndex = IRT_Contains(hunted, unitName);
		local group = getGroup(unitName); 
		if (UnitIsUnit(leader, playerName)) then
			if (not plIndex and IRT_UnitDebuff(unit, GetSpellInfo(361745))) then
				table.insert(hunted, unitName);
				if (#hunted == 4) then
					assignments();
				end
			elseif (plIndex and not IRT_UnitDebuff(unit, GetSpellInfo(361745))) then 
				table.remove(hunted, plIndex);
				raid[group][unitName].hunted = false;
			end
			if (not raid[group][unitName].debuffed and IRT_UnitDebuff(unit, GetSpellInfo(361784))) then
				raid[group][unitName].debuffed = true;
			elseif (raid[group][unitName].debuffed and not IRT_UnitDebuff(unit, GetSpellInfo(361784))) then 
				raid[group][unitName].hunted = false;
			end
		end
	elseif (event == "ENCOUNTER_START" and IRT_PrototypePantheonEnabled) then
		local eID = ...;
		local difficulty = select(3, GetInstanceInfo());
		if (eID == 2544 and difficulty == 16) then
			inEncounter = true;
			initRaid();
			debuffed = {};
			hunted = {};
			leader = IRT_GetRaidLeader();
		end
	elseif (event == "ENCOUNTER_END" and inEncounter and IRT_PrototypePantheonEnabled) then
		inEncounter = false;
		debuffed = {};
		hunted = {};
	end
end);